% Copyright 2019. All Rights Reserved
% Code by Shuang Li
% For Paper, "The Landscape of Non-convex Empirical Risk with Degenerate Population Risk"
% Advances in Neural Information Processing Systems, 2019
% by Shuang Li, Gongguo Tang, and Michael B. Wakin
% This code is used to reproduce Figure 6.


%%

clc; clear; close all;

MM = 10:10:100;     % number of samples
N = 8;            % signal dimension
r = 3;            % rank of data matrix X
k = 2;            % # of cols in U0, U
Ustar = eye(N); Ustar = Ustar(:,1:r);
X = Ustar*Ustar';

%% ----------- compute local minima with GD -----------
U0 = Ustar(:,1:k);
numIter = 1e2;
stepsize = 0.01;
Trial = 100;
dist = zeros(1,length(MM));
for t = 1:Trial
    fprintf('Trial = %d\n',t)
    Ug_local = zeros(N,k,length(MM));
    Uf_local = zeros(N,k,length(MM));
    for m = 1:length(MM)
        M = MM(m);
        
        % population risk
        U = U0;
        cost_g = zeros(1,numIter);
        for i = 1:numIter
            grad = (U*U'-X)*U;
            U = U - stepsize*grad;
            cost_g(i) = norm(U*U'-X,'fro')^2/4;
        end
        Ug_local(:,:,m) = U;
        
        % empirical risk
        B = randn(N,N,M)/sqrt(M);
        U = U0;
        cost_f = zeros(1,numIter);
        for i = 1:numIter
            grad = zeros(size(U));
            for j = 1:M
                Am = (B(:,:,j)+B(:,:,j)')/2;
                grad = grad + Am*U*(vec(Am)'*vec(U*U'-X));
            end
            U = U - stepsize*grad;
            for j = 1:M
                Am = (B(:,:,j)+B(:,:,j)')/2;
                cost_f(i) = cost_f(i) + (vec(Am)'*vec(U*U'-X))^2;
                cost_f(i) = cost_f(i)/4;
            end
        end
        Uf_local(:,:,m) = U;
        [~,SS] = procrustes(Ug_local(:,:,m),Uf_local(:,:,m));
        dist(m) = dist(m) + norm(Ug_local(:,:,m)-SS,'fro');
    end
end
dist = dist/Trial;
%%

figure
set(gcf,'position',[10,200,420,280])
fs = 22;
lw = 3;
ms = 10;
plot(MM,dist,'r-o','linewidth',lw,'markersize',ms);
xlabel('$M$','Interpreter','LaTex','FontSize',fs)
ylabel('Estimation error','FontSize',fs)
set(gca,'fontsize',fs)
axis tight;
set(gcf, 'Color', 'w');















