% Copyright 2019. All Rights Reserved
% Code by Shuang Li
% For Paper, "The Landscape of Non-convex Empirical Risk with Degenerate Population Risk"
% Advances in Neural Information Processing Systems, 2019
% by Shuang Li, Gongguo Tang, and Michael B. Wakin
% This code is used to reproduce Figure 1.


%% 1D phase retrieval

clc; clear; %close all;

x = linspace(-1.5,1.5,100);

M = 30;            % # of measurements
a = randn(M,1);     % random sensing vector
xs = 1;             % xstar: true signal

g = 1.5*(x.^2-xs^2).^2;    % population risk
f = sum(a.^4).*(x.^2-xs^2).^2/2/M;    % empirical risk

grad_g = 6*x.^3-6*xs^2*x;
grad_f = sum(a.^4)*(x.^3-xs^2*x)*2/M;
hess_g = 18*x.^2-6*xs^2;
hess_f = sum(a.^4)*(3*x.^2-xs^2)*2/M;

indx = find(abs(grad_g)<=0.5);
x_smallgrad = x(indx);

%%

figure(1)
clf;
fs = 24;
lw = 3;
set(gcf,'position',[10,50,560,610])
subplot(3,1,1)
plot(x,g,'r-','linewidth',lw);hold on;
plot(x,f,'b--','linewidth',lw);hold on;
h=legend('Population','Empirical');
ylabel('Risks','FontSize',fs)
set(gca,'fontsize',fs)
axis tight;
subplot(3,1,2)
plot(x,grad_g,'r-','linewidth',lw);hold on;
plot(x,grad_f,'b--','linewidth',lw);hold on;
%plot(x,zeros(1,length(x)),'k-','linewidth',lw-1); hold on;
plot(x,1*ones(1,length(x)),'k-','linewidth',lw-1); hold on;
plot(x,-1*ones(1,length(x)),'k-','linewidth',lw-1); hold on;
%plot(x_smallgrad,'r*'); hold on;
ylabel('Gradients','FontSize',fs)
yticks([-10 -1 1 10])
set(gca,'fontsize',fs)
axis tight;
subplot(3,1,3)
plot(x,hess_g,'r-','linewidth',lw);hold on;
plot(x,hess_f,'b--','linewidth',lw);hold on;
plot(x,zeros(1,length(x)),'k-','linewidth',lw-1); hold on;
xlabel('$x$','Interpreter','LaTex','FontSize',fs)
ylabel('Hessians','FontSize',fs)
set(h,'fontsize',fs-2)
set(gca,'fontsize',fs)
set(gcf, 'Color', 'w');
axis tight;
